@@ -6,11 +6,11 @@ from importlib.resources import files
 from termcolor import colored, cprint
 import time
 import shutil 
-from .typings import Command
+from .typings import Command, BlocklessNodeType
 import re
 import yaml
 
-def create_worker_account(worker_name, faucet_url, type='worker'):
+def create_worker_account(worker_name, faucet_url, type):
     current_file_dir = os.path.dirname(os.path.abspath(__file__))
     cli_tool_dir = os.path.dirname(current_file_dir)
     allora_chain_dir = os.path.join(cli_tool_dir, 'allora-chain')
@@ -78,7 +78,7 @@ def create_worker_account(worker_name, faucet_url, type='worker'):
                         f'{faucet_url}/send/testnet/{address}'
                     ], stdout=subprocess.DEVNULL)
         
-        print(colored(f"keys created and testnet-funded for this worker. please check config.yaml for your address and mnemonic", "green"))
+        print(colored(f"keys created and testnet-funded for this {type}. please check config.yaml for your address and mnemonic", "green"))
         return mnemonic, hex_coded_pk, address
     else:
         print(colored("'make' is not available in the system's PATH. Please install it or check your PATH settings.", "red"))
@@ -100,18 +100,18 @@ def print_allora_banner():
     """
     cprint(banner_text, 'blue', attrs=['bold'])
 
-def generate_all_files(env: Environment, file_configs, command: Command, worker_name = ''):
+def generate_all_files(env: Environment, file_configs, command: Command, type, name = ''):
     if command == Command.INIT:
-        cprint(f"Bootstraping '{worker_name}' directory...", 'cyan')
+        cprint(f"Bootstraping '{name}' directory...", 'cyan')
         time.sleep(1) 
 
     for config in file_configs:
         template = env.get_template(config["template_name"])
 
         if command == Command.INIT:
-            file_path = os.path.join(os.getcwd(), f'{worker_name}/{config["file_name"]}')
+            file_path = os.path.join(os.getcwd(), f'{name}/{type}/{config["file_name"]}')
         elif command == Command.DEPLOY: 
-            file_path = os.path.join(os.getcwd(), f'{config["file_name"]}')
+            file_path = os.path.join(os.getcwd(), f'{type}/{config["file_name"]}')
 
         content = template.render(**config["context"])
         with open(file_path, 'w') as f:
@@ -120,24 +120,24 @@ def generate_all_files(env: Environment, file_configs, command: Command, worker_
     if command == Command.INIT:
         cprint("\nAll files bootstrapped successfully. ALLORA!!!", 'green', attrs=['bold'])
 
-def run_key_generate_command(worker_name):
+def run_key_generate_command(worker_name, type):
     command = (
-        f'docker run -it --entrypoint=bash -v "$(pwd)/{worker_name}/data":/data '
+        f'docker run -it --entrypoint=bash -v "$(pwd)/{worker_name}/{type}/data":/data '
         'alloranetwork/allora-inference-base:latest '
-        '-c "mkdir -p /data/head/key /data/worker/key && (cd /data/head/key && allora-keys) && (cd /data/worker/key && allora-keys)"'
+        f'-c "mkdir -p /data/head/key /data/{type}/key && (cd /data/head/key && allora-keys) && (cd /data/{type}/key && allora-keys)"'
     )
     try:
         subprocess.run(command, shell=True, check=True)
-        peer_id_path = os.path.join(os.getcwd(), f'{worker_name}/data/head/key', 'identity')
+        peer_id_path = os.path.join(os.getcwd(), f'{worker_name}/{type}/data/head/key', 'identity')
         with open(peer_id_path, 'r') as file:
-            cprint(f"local workers identity generated successfully.", 'cyan')
+            cprint(f"local {type} identity generated successfully.", 'cyan')
             head_peer_id = file.read().strip()
             return head_peer_id
     except subprocess.CalledProcessError as e:
-        click.echo(f"error generating local workers identity: {e}", err=True)
+        click.echo(f"error generating local {type} identity: {e}", err=True)
 
-def generateWorkerAccount(worker_name):
-    config_path = os.path.join(os.getcwd(), worker_name, 'config.yaml')
+def generateWorkerAccount(worker_name, type):
+    config_path = os.path.join(os.getcwd(), worker_name, type, 'config.yaml')
     try:
         with open(config_path, 'r') as file:
             config = yaml.safe_load(file)
@@ -148,62 +148,162 @@ def generateWorkerAccount(worker_name):
     worker_name = config['name']
     faucet_url = config['faucet_url']
     account_details = None
-    if not config['worker']['mnemonic'] or not config['worker']['hex_coded_pk'] or not config['worker']['address']:
-        account_details = create_worker_account(worker_name, faucet_url, 'worker')
+    if not config[type]['mnemonic'] or not config[type]['hex_coded_pk'] or not config[type]['address']:
+        account_details = create_worker_account(worker_name, faucet_url, type)
 
-    mnemonic = account_details[0] if account_details else config['worker']['mnemonic']
-    hex_coded_pk = account_details[1] if account_details else config['worker']['hex_coded_pk']
-    address = account_details[2] if account_details else config['worker']['address']
+    mnemonic = account_details[0] if account_details else config[type]['mnemonic']
+    hex_coded_pk = account_details[1] if account_details else config[type]['hex_coded_pk']
+    address = account_details[2] if account_details else config[type]['address']
 
-    if not config['worker']['mnemonic'] or not config['worker']['hex_coded_pk'] or not config['worker']['address']:
-        config['worker']['mnemonic'] = mnemonic
-        config['worker']['hex_coded_pk'] = hex_coded_pk
-        config['worker']['address'] = address
+    if not config[type]['mnemonic'] or not config[type]['hex_coded_pk'] or not config[type]['address']:
+        config[type]['mnemonic'] = mnemonic
+        config[type]['hex_coded_pk'] = hex_coded_pk
+        config[type]['address'] = address
         with open(config_path, 'w') as file:
             yaml.safe_dump(config, file)
 
-def generateProdCompose(env: Environment):
+def generateProdCompose(env: Environment, type):
     """Deploy resource production kubernetes cluster"""
 
-    command = "mkdir -p ./scripts"
-    subprocess.run(command, shell=True, check=True)
+    cprint(f"\nMake sure you are running this command in the appropriate directory [validator, reputer, worker]", 'cyan')
+    cprint(f"\nif not, please cd to the right directory", 'cyan')
+    if click.confirm(colored("\nif you are in the right folder, please proceed", 'white', attrs=['bold']), default=True):
 
-    config_path = os.path.join(os.getcwd(), 'config.yaml')
-    try:
-        with open(config_path, 'r') as file:
-            config = yaml.safe_load(file)
-    except yaml.YAMLError as e:
-        print(colored(f"Error reading config file: {e}", 'red', attrs=['bold']))
+        subprocess.run("mkdir -p ./data/scripts", shell=True, check=True)
+
+        try:
+            result = subprocess.run("chmod -R +rx ./data/scripts", shell=True, check=True, capture_output=True, text=True)
+            print(result)
+        except subprocess.CalledProcessError as e:
+            print(f"Command '{e.cmd}' returned non-zero exit status {e.returncode}.")
+            if e.stderr:
+                print(f"Stderr: {e.stderr}")
+
+        config_path = os.path.join(os.getcwd(), 'config.yaml')
+        try:
+            with open(config_path, 'r') as file:
+                config = yaml.safe_load(file)
+        except yaml.YAMLError as e:
+            print(colored(f"Error reading config file: {e}", 'red', attrs=['bold']))
+            return
+
+        worker_name = config['name']
+        hex_coded_pk = config[type]['hex_coded_pk']
+        boot_nodes = config[type]['boot_nodes']
+        chain_rpc_address = config[type]['chain_rpc_address']
+        chain_topic_id = config[type]['chain_topic_id']
+
+        file_configs = [
+            {
+                "template_name": "prod-docker-compose.yaml.j2",
+                "file_name": "prod-docker-compose.yaml",
+                "context": {
+                    "worker_name": worker_name, 
+                    "boot_nodes": boot_nodes, 
+                    "chain_rpc_address": chain_rpc_address, 
+                    "topic_id": chain_topic_id, 
+                }
+            },
+            {
+                "template_name": "init.sh.j2",
+                "file_name": "data/scripts/init.sh",
+                "context": {
+                    "worker_name": worker_name, 
+                    "hex_coded_pk": hex_coded_pk
+                }
+            }
+        ]
+
+        generate_all_files(env, file_configs, Command.DEPLOY)
+        cprint(f"production docker compose file generated to be deployed", 'green')
+        # cprint(f"please run chmod -R +rx ./data/scripts to grant script access to the image", 'yellow')
+    else:
+        cprint("\nOperation cancelled.", 'red')
+
+
+def blocklessNode(environment, env, type, name=None, topic=None):
+    """Initialize your Allora Worker Node with necessary boilerplates"""
+
+    if not check_docker_running():
+        cprint("Docker is not running on your machine, please start docker before running this command", 'red')
         return
+    
+    if environment == 'dev':
+        if topic is None:
+            cprint(f"You must provide topic id when generating {type} in development", 'red')
+            return
+        elif name is None:
+            cprint(f"You must provide name when generating {type} in development", 'red')
+            return
+
+        print_allora_banner()
+        cprint("Welcome to the Allora CLI!", 'green', attrs=['bold'])
+        print(colored(f"Allora CLI assists in the seamless creation and deployment of Allora {type} nodes", 'yellow'))
+        cprint(f"\nThis command will generate some files in the directory named '{name}'.", 'cyan')
+        
+        if click.confirm(colored("\nWould you like to proceed?", 'white', attrs=['bold']), default=True):
+            cprint(f"\nProceeding with the creation of {type} node directory...", 'green')
+
+            head_peer_id = run_key_generate_command(name, type)
+
+            file_configs = [
+                {
+                    "template_name": "Dockerfile.j2",
+                    "file_name": "Dockerfile",
+                    "context": {}
+                },
+                {
+                    "template_name": "main.py.j2",
+                    "file_name": "main.py",
+                    "context": {}
+                },
+                {
+                    "template_name": "dev-docker-compose.yaml.j2",
+                    "file_name": "dev-docker-compose.yaml",
+                    "context": {"head_peer_id": head_peer_id, "topic_id": topic, "b7s_type": type}
+                },
+                {
+                    "template_name": "requirements.txt.j2",
+                    "file_name": "requirements.txt",
+                    "context": {}
+                },
+                {
+                    "template_name": "gitignore.j2",
+                    "file_name": ".gitignore",
+                    "context": {}
+                },
+                {
+                    "template_name": "env.j2",
+                    "file_name": ".env",
+                    "context": {}
+                },
+                {
+                    "template_name": "config.yaml.j2",
+                    "file_name": "config.yaml",
+                    "context": {"name": name, "topic_id": topic, "b7s_type": type}
+                }
+            ]
+
+            generate_all_files(env, file_configs, Command.INIT, name)
+
+            generateWorkerAccount(name, type)
+        else:
+            cprint("\nOperation cancelled.", 'red')
+    elif environment == 'prod':
+        devComposePath = os.path.join(os.getcwd(), type, 'dev-docker-compose.yaml')
+        if not os.path.exists(devComposePath):
+            cprint(f"You must initialize the {type} on dev please run allocmd generate {type} --env dev --name <{type} name> --topic <topic id> and then run the prod generate in the directory created", 'red')
+        else:
+            generateProdCompose(env, type)
+
+
+
+
+
+
 
-    worker_name = config['name']
-    hex_coded_pk = config['worker']['hex_coded_pk']
-    boot_nodes = config['worker']['boot_nodes']
-    chain_rpc_address = config['worker']['chain_rpc_address']
-    chain_topic_id = config['worker']['chain_topic_id']
 
-    file_configs = [
-        {
-            "template_name": "prod-docker-compose.yaml.j2",
-            "file_name": "prod-docker-compose.yaml",
-            "context": {
-                "worker_name": worker_name, 
-                "boot_nodes": boot_nodes, 
-                "chain_rpc_address": chain_rpc_address, 
-                "topic_id": chain_topic_id, 
-            }
-        },
-        {
-            "template_name": "init.sh.j2",
-            "file_name": "scripts/init.sh",
-            "context": {
-                "worker_name": worker_name, 
-                "hex_coded_pk": hex_coded_pk
-            }
-        }
-    ]
 
-    generate_all_files(env, file_configs, Command.DEPLOY)
 
 def deployWorker(env: Environment):
     """Deploy resource production kubernetes cluster"""
@@ -406,3 +506,11 @@ def deployValidator(env: Environment):
             return
     else:
         print(colored('Operation cancelled.', 'magenta'))
+
+def check_docker_running():
+    """Check if Docker daemon is running."""
+    try:
+        subprocess.run(['docker', 'info'], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL, check=True)
+        return True
+    except subprocess.CalledProcessError:
+        return False
